var FEEDBAR = {
	get tree() { return document.getElementById("feedbar_tree"); },
	get progressText() { return document.getElementById("feedbar-loading-text"); },
	get previewPane() { return document.getElementById("feedbar-preview"); },
	get nextUpdateDisplay() { return document.getElementById("feedbar-nextupdate-text"); },
	get strings() { return document.getElementById("feedbar-string-bundle"); },
	get displayPeriod() { return this.prefs.getIntPref("displayPeriod"); },
	
	prefs : null,
	
	feedsToLoad : 0,
	feedsLoaded : 0,
	
	updateTimer : null,
	loadTimer : null,
	currentRequest : null,
	
	feeds : [],
	
	get lastUpdate() { 
		// Stored as the number of seconds since the epoch
		// Should reveal this value as a JavaScript Date object
		
		var secondsBetween, timestamp = 0;
		var updateObj = new Date();
		
		try {
			timestamp = this.prefs.getIntPref("lastUpdate");
		} catch (e) {
			secondsBetween = this.prefs.getIntPref("updateFrequency") * 60;
			timestamp = Math.round(updateObj.getTime() / 1000);
			timestamp -= secondsBetween;
		}
		
		updateObj.setTime(timestamp * 1000);
		
		return updateObj;
	},
	
	set lastUpdate(dateOb) {
		var timestamp = Math.round(dateOb.getTime() / 1000);
		this.prefs.setIntPref("lastUpdate", timestamp);
	},
	
	get nextUpdate() { 
		var lastUpdate = this.lastUpdate;
		var milliBetween = this.prefs.getIntPref("updateFrequency") * 60 * 1000;
		var lastMilli = lastUpdate.getTime();
		var nextUpdate = new Date();
		nextUpdate.setTime(lastUpdate.getTime() + milliBetween);
		
		return nextUpdate;
	},
	
	init : function () {
		this.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.feedbar.");	
		this.strings = document.getElementById("feedbar-string-bundle");
		
		this.updateLoadProgress(0,0);
		this.checkFrequencyItem(this.prefs.getIntPref("updateFrequency"));
		this.checkPeriodItem(this.prefs.getIntPref("displayPeriod"));	
		
		// Update the display from the stored feeds
		this.updateFeeds(window.parent.FEEDBAR_STORE);
	
		// The sidebar has been opened and closed at least once.
		var lastMilli = this.lastUpdate.getTime();
		var currentMilli = new Date().getTime();
		var milliBetween = this.prefs.getIntPref("updateFrequency") * 60 * 1000;
		
		if ((lastMilli + milliBetween) < currentMilli){
			// Only do an update if enough time has elapsed since the sidebar was closed
			this.updateFeeds();
		}
		else {
			// Set an update for when it would have happened had the sidebar stayed open.
			this.updateTimer = setTimeout("FEEDBAR.updateFeeds();", (lastMilli + milliBetween) -  currentMilli);
		}
		
		this.previewPane.style.visibility = 'hidden';
	},
	
	updateFeeds : function (feeds) {
		if (feeds) {
			// Use the supplied array of feed objects to update the display
			for (var i = 0; i < feeds.length; i++){
				FEEDBAR.showItems(feeds[i]);
			}
		}
		else {
			// Find and load the feeds
			if (this.updateTimer) window.clearTimeout(this.updateTimer);
			
			this.findFeeds();
			this.loadFeeds();
			
			this.updateTimer = setTimeout("FEEDBAR.updateFeeds();", this.prefs.getIntPref("updateFrequency") * 60 * 1000);
		}
	},
	
	stopUpdate : function () {
		while (this.feeds.length > 0) this.feeds.pop();
		this.loadNextFeed();
	},
	
	findFeeds : function () {
		if (!RDF) initServices();
		if (!BMSVC) initBMService();
		
		var root = RDF.GetResource("NC:BookmarksRoot");
		var feedURLArc = RDF.GetResource("http://home.netscape.com/NC-rdf#FeedURL");
		var urlArc = RDF.GetResource("http://home.netscape.com/NC-rdf#URL");
		var nameArc = RDF.GetResource("http://home.netscape.com/NC-rdf#Name");
		
		var folders = [ root ];
		
		while (this.feeds.length > 0) this.feeds.pop();
		
		while (folders.length > 0){
			RDFC.Init(BMDS, folders.shift());
		
			var elements = RDFC.GetElements();
		
			while(elements.hasMoreElements()) {
				var element = elements.getNext();
				element.QueryInterface(Components.interfaces.nsIRDFResource);
		
				var type = BookmarksUtils.resolveType(element);
				
				if ((type == "Folder") || (type == "PersonalToolbarFolder")){
					folders.push(element);
				}
				else if (type == 'Livemark') {
					var res = RDF.GetResource(element.Value);
					var target = BMDS.GetTarget(res, feedURLArc, true);
					
					if (target) {
						var feedURL = target.QueryInterface(kRDFLITIID).Value;
						
						var target = BMDS.GetTarget(res, urlArc, true);
						
						if (target) {
							var siteURL = target.QueryInterface(kRDFLITIID).Value;
						}
						else {
							var siteURL = 'about:blank';
						}
						
						var target = BMDS.GetTarget(res, nameArc, true);
						
						if (target) {
							var feedName = target.QueryInterface(kRDFLITIID).Value;
						}
						else {
							var feedName = feedURL;
						}
						
						this.feeds.push({ name : feedName, feed : feedURL, site : siteURL });
					}
				}
			}
		}
		
		this.feedsToLoad = this.feeds.length;
		this.feedsLoaded = 0;
	},
	
	loadFeeds : function () {
		this.lastUpdate = new Date();
		
		FEEDBAR.clearNotify();
		while (window.parent.FEEDBAR_STORE.length > 0) window.parent.FEEDBAR_STORE.pop();
		
		if (this.feedsToLoad == 0){
			FEEDBAR.notifyNoFeeds();
		}
		
		FEEDBAR.updateLoadProgress(0, this.feedsToLoad);
		this.loadNextFeed();
	},
	
	loadNextFeed : function () {
		var feed = this.feeds.shift();
		
		if (feed){
			var url = feed.feed;
			this.progressText.setAttribute("tooltiptext",url);
			var req = new XMLHttpRequest();
			req.parent = this;
			req.feed = feed;
			this.currentRequest = req;
			
			try {
				req.open("GET", url, true);
				req.overrideMimeType("application/xml");
				
				req.onreadystatechange = function (event) {
					if (req.readyState == 4) {
						clearTimeout(req.parent.loadTimer);
						
						try {
							if (req.status == 200){
								var feedOb = null;
								
								try {
									feedOb = new FeedbarFeed(req.responseXML, url);
									FEEDBAR.showItems(feedOb);
									window.parent.FEEDBAR_STORE.push(feedOb);
								} catch (e) {
									// Parse error
									FEEDBAR.addError(feed.name, url, e.message, 5);
								}
							}
						}
						catch (e) {
							if (e.name == "NS_ERROR_NOT_AVAILABLE"){
								FEEDBAR.addError(feed.name, url, FEEDBAR.strings.getString("feedbar.errors.unavailable"), 3);
							}
						}
						
						FEEDBAR.updateLoadProgress(++req.parent.feedsLoaded, req.parent.feedsToLoad);
						req.parent.loadNextFeed();
					}
				};
				
				req.send(null);
				this.loadTimer = setTimeout('FEEDBAR.currentRequest.abort();', 1000 * 15);
			}
			catch (e) {
				FEEDBAR.addError(feed.name, url, e.name, 3);
				FEEDBAR.updateLoadProgress(++this.feedsLoaded, this.feedsToLoad);
				this.loadNextFeed();
			}
		}
		else {
			FEEDBAR.updateLoadProgress(0, 0);
		}
	},
	
	setDisplayPeriod : function (days) {
		this.prefs.setIntPref("displayPeriod", days);
		
		this.clearTree();
		this.updateFeeds(window.parent.FEEDBAR_STORE);
	},
	
	clearTree : function () {
		var c = document.getElementById('feedbar_tree_container');
		
		while (c.lastChild) c.removeChild(c.lastChild);
	},
	
	setReloadInterval : function (minutes) {
		minutes = parseInt(minutes);
		minutes = Math.max(minutes, 1);

		if (minutes != this.prefs.getIntPref("updateFrequency")) {
			var lastUpdate = this.lastUpdate;
			
			this.prefs.setIntPref("updateFrequency",minutes);
			window.clearTimeout(this.updateTimer);
			
			var newMSBetween = minutes * 60 * 1000;
			
			var newNextUpdateTime = new Date();
			newNextUpdateTime.setTime(lastUpdate.getTime() + newMSBetween);
			
			var now = new Date();
			
			if (newNextUpdateTime.getTime() <= now.getTime()) {
				FEEDBAR.updateFeeds();
			}
			else {
				var msUntil = newNextUpdateTime.getTime() - now.getTime();
				FEEDBAR.updateLoadProgress(0,0);
				this.updateTimer = setTimeout("FEEDBAR.updateFeeds();", msUntil);
			}
		}
	},
	
	checkFrequencyItem : function (minutes) {
		var frequencyMenu = document.getElementById('frequency-menu');
		var frequencies = frequencyMenu.getElementsByTagName("menuitem");
		
		for (var i = 0; i < frequencies.length; i++){
			if (frequencies[i].getAttribute("value") == minutes){
				frequencies[i].setAttribute("checked","true");
			}
			else {
				frequencies[i].setAttribute("checked","false");
			}
		}
	},
	
	checkPeriodItem : function (days) {
		var periodMenu = document.getElementById('period-menu');
		var periods = periodMenu.getElementsByTagName("menuitem");
		
		for (var i = 0; i < periods.length; i++){
			if (periods[i].getAttribute("value") == days){
				periods[i].setAttribute("checked","true");
			}
			else {
				periods[i].setAttribute("checked","false");
			}
		}
	},
	
	history : {
		hService : Components.classes["@mozilla.org/browser/global-history;2"].getService(Components.interfaces.nsIGlobalHistory2),
		ioService : Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService),
		
		URI : null,
		
		addToHistory : function (url, title) {
			try {
				this.URI = this.ioService.newURI(url, null, null);
				this.hService.addURI(this.URI, false, true, null);
				this.hService.setPageTitle(this.URI, title);
			} catch (e) {
			}
		},
		
		isVisitedURL : function(url){
			try {
				this.URI = this.ioService.newURI(url, null, null);
				return this.hService.isVisited(this.URI);
			} catch (e) {
				// Malformed URI, probably
				return false;
			}
		}
	},

	updateLoadProgress : function (done, total) {
		if (done == total) {
			var nextUpdateTime = this.nextUpdate;
			
			var timeText = '';
			timeText += ((nextUpdateTime.getHours() > 12) ? nextUpdateTime.getHours() - 12 : nextUpdateTime.getHours());
			
			if (timeText == "0") timeText = "12";
			timeText += ":";
			
			if (nextUpdateTime.getMinutes() < 10) timeText += "0";
			timeText += nextUpdateTime.getMinutes();
			timeText += " ";
			
			timeText += (nextUpdateTime.getHours() > 11) ? this.strings.getString("feedbar.time.pm") : this.strings.getString("feedbar.time.am");
			
			document.getElementById("feedbar-reload-button").setAttribute("disabled","false");
			document.getElementById("feedbar-stop-button").setAttribute("disabled","true");
			this.progressText.setAttribute("value", this.strings.getFormattedString("feedbar.nextUpdate", [timeText]) );
		}
		else {
			document.getElementById("feedbar-reload-button").setAttribute("disabled","true");
			document.getElementById("feedbar-stop-button").setAttribute("disabled","false");
			this.progressText.setAttribute("value", this.strings.getFormattedString("feedbar.checkingFeeds", [done, total]));
		}
	},
		
	itemSelect : function (event) {
		var target = this.getSelected();
		this.showPreview(target);
	},
	
	onTreeClick : function (event, target) {
		// Discard right-clicks
		if (event.which == 3){
			return;
		}
		
		if (!target) target = this.getSelected();
		
		if (target && target.type == 'item'){
			if (event.which == 2){
				this.launchUrl(target.link, event);
				this.removeItem(target);
			}
			else if (event.which == 4){
				window.open(target.link);
			}
			else {
				// Left-click
				if (event.detail == 1){
					// Show the details
				//	this.showPreview(target);
				// handled by onselect
				}
				else {
					this.launchUrl(target.link, event);
					this.removeItem(target);
				}
			}
		}
	},
	
	removeItem : function (item) {
		var container = item.parentNode;
		
		if (container.childNodes.length == 1){
			container.parentNode.parentNode.removeChild(container.parentNode);
		}
		else {
			container.removeChild(item);
		}
		
		if (this.tree.getElementsByTagName("treeitem").length == 0) window.parent.document.getElementById("sidebar").style.width = '5px;';
	},
	
	showItems : function(feedObject) {
		var t = document.getElementById("feedbar_tree_container");
		
		var foundOne = false;
		
		var items = feedObject.getItems();
		
		itemsAdd : for (var k = 0; k < items.length; k++){
			if (!FEEDBAR.history.isVisitedURL(items[k].link)){
				if (this.displayPeriod > 0) {
					var itemTime = Date.parse(items[k].pubDate);
					var mustBeAfter = (new Date()).getTime() - (this.displayPeriod * 24 * 60 * 60 * 1000);
					
					if (itemTime < mustBeAfter) {
						continue itemsAdd;
					}
				}
				
				if (!foundOne) {
					foundOne = true;
					
					for (var q = 0; q < t.childNodes.length; q++){
						if (t.childNodes[q].feed == feedObject.feedURL){
							var fp = t.childNodes[q].getElementsByTagName("treechildren")[0];
						}
					}
					
					if (!fp){
						var f = document.createElement("treeitem");
						f.link = (feedObject.link) ? feedObject.link : feedObject.feedURL;
						f.title = feedObject.title;
						f.description = feedObject.description;
						f.image = feedObject.rootLink + "favicon.ico";
						f.feedName = feedObject.title;
						f.type = 'feed';
						
						f.feed = feedObject.feedURL;
						f.setAttribute("container","true");
						f.setAttribute("open","true");
						
						var fr = document.createElement("treerow");
						var fc = document.createElement("treecell");
						fc.setAttribute("src","chrome://feedbar/skin/icons/folder.png");
						fc.setAttribute("label"," "+feedObject.title);
						
						var fp = document.createElement("treechildren");
						
						t.appendChild(f);
						f.appendChild(fr);
						fr.appendChild(fc);
						f.appendChild(fp);
					}
				}
				
				itemsCheck : for (var q = 0; q < fp.childNodes.length; q++){
					if (fp.childNodes[q].link == items[k].link) {
						continue itemsAdd;
					}
				}
				
				var i = document.createElement("treeitem");
				
				var r = document.createElement("treerow");
				var c = document.createElement("treecell");
				c.setAttribute("src",feedObject.rootLink + "favicon.ico");
				c.setAttribute("label"," "+items[k].title);
				
				i.link = items[k].link;
				i.title = items[k].title;
				i.description = items[k].content;
				i.image = feedObject.rootLink + "favicon.ico";
				i.feedName = feedObject.title;
				i.type = 'item';
				i.setAttribute("tooltip","feedbarTooltip");
				
				t.appendChild(i);
				i.appendChild(r);
				r.appendChild(c);
				
				fp.appendChild(i);
			}
		}
	},
	
	contextMenu : {
		get item() { return FEEDBAR.getSelected(); },
		
		customize : function (menu) {
			var options = menu.getElementsByTagName("menuitem");
			var item = FEEDBAR.getSelected();
			
			if (item) {
				if (item.type == 'item') {
					// Single item menu
					for (var i = 0; i < options.length; i++){
						if (options[i].getAttribute("option") == "open") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "openInWindow") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "openInTab") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "openAllInTabs") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "openFeedInTabs") {
							options[i].setAttribute("hidden","true");
						}
						if (options[i].getAttribute("option") == "refreshFeeds") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "markAsRead") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "markFeedAsRead") {
							options[i].setAttribute("hidden","true");
						}
						if (options[i].getAttribute("option") == "markAllAsRead") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "options") {
							options[i].setAttribute("hidden","false");
						}
					}	
				}
				else {
					// Feed menu
					for (var i = 0; i < options.length; i++){
						if (options[i].getAttribute("option") == "open") {
							options[i].setAttribute("hidden","true");
						}
						if (options[i].getAttribute("option") == "openInWindow") {
							options[i].setAttribute("hidden","true");
						}
						if (options[i].getAttribute("option") == "openInTab") {
							options[i].setAttribute("hidden","true");
						}
						if (options[i].getAttribute("option") == "openAllInTabs") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "openFeedInTabs") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "refreshFeeds") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "markAsRead") {
							options[i].setAttribute("hidden","true");
						}
						if (options[i].getAttribute("option") == "markFeedAsRead") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "markAllAsRead") {
							options[i].setAttribute("hidden","false");
						}
						if (options[i].getAttribute("option") == "options") {
							options[i].setAttribute("hidden","false");
						}
					}	
				}
			}
			else {
				// Default menu
				for (var i = 0; i < options.length; i++){
					if (options[i].getAttribute("option") == "open") {
						options[i].setAttribute("hidden","true");
					}
					if (options[i].getAttribute("option") == "openInWindow") {
						options[i].setAttribute("hidden","true");
					}
					if (options[i].getAttribute("option") == "openInTab") {
						options[i].setAttribute("hidden","true");
					}
					if (options[i].getAttribute("option") == "openAllInTabs") {
						options[i].setAttribute("hidden","false");
					}
					if (options[i].getAttribute("option") == "openFeedInTabs") {
						options[i].setAttribute("hidden","true");
					}
					if (options[i].getAttribute("option") == "refreshFeeds") {
						options[i].setAttribute("hidden","false");
					}
					if (options[i].getAttribute("option") == "markAsRead") {
						options[i].setAttribute("hidden","true");
					}
					if (options[i].getAttribute("option") == "markFeedAsRead") {
						options[i].setAttribute("hidden","true");
					}
					if (options[i].getAttribute("option") == "markAllAsRead") {
						options[i].setAttribute("hidden","false");
					}
					if (options[i].getAttribute("option") == "options") {
						options[i].setAttribute("hidden","false");
					}
				}	
			}
			
			var foundOne = false;
			
			for (var i = 0; i < menu.childNodes.length; i++){
				if ((menu.childNodes[i].localName == "menuitem") && (menu.childNodes[i].getAttribute("hidden") == "false")){
					foundOne = true;
				}
				else if (menu.childNodes[i].localName == "menuseparator"){
					if (foundOne) {
						menu.childNodes[i].setAttribute("hidden","false");
						foundOne = false;
					}
					else {
						menu.childNodes[i].setAttribute("hidden","true");
					}
				}
			}
			
			return true;
		},
		
		openItem : function () {
			FEEDBAR.onTreeCblClick({which : 1, detail : 2});
		},
		
		openInWindow : function () {
			FEEDBAR.onTreeClick({which : 4, detail : 2});
		},
		
		openInTab : function () {
			FEEDBAR.onTreeClick({which : 2, detail : 1});
		},
		
		openAll : function () {
			var items = FEEDBAR.tree.getElementsByTagName("treeitem");
			
			for (var i = items.length - 1; i >= 0; i--){
				if (items[i] && items[i].type == 'item') {
					FEEDBAR.onTreeClick({which : 2, detail : 1}, items[i]);
				}
			}
		},
		
		openFeed : function () {
			var items = this.item.getElementsByTagName("treeitem");
			
			for (var i = items.length - 1; i >= 0; i--){
				if (items[i].type == 'item') {
					FEEDBAR.onTreeClick({which : 2, detail : 1}, items[i]);
				}
			}
		},
		
		markAsRead : function () {
			if (this.item.type == 'item'){
				FEEDBAR.history.addToHistory(this.item.link, this.item.title);
				FEEDBAR.removeItem(this.item);
			}
			else if (this.item.type == 'feed'){
				this.markFeedAsRead();
			}
		},
		
		markFeedAsRead : function () {
			var items = this.item.getElementsByTagName("treeitem");
			
			for (var i = items.length - 1; i >= 0; i--){
				if (items[i].type == 'item') {
					FEEDBAR.history.addToHistory(items[i].link, items[i].title);
					FEEDBAR.removeItem(items[i]);
				}
			}
		},
		
		markAllAsRead : function () {
			var container = document.getElementById("feedbar_tree_container");
			
			var item = null;
			
			while ((item = container.lastChild)) {
				// item is a feed
				var items = item.getElementsByTagName("treeitem");
			
				for (var i = items.length - 1; i >= 0; i--){
					if (items[i].type == 'item') {
						FEEDBAR.history.addToHistory(items[i].link, items[i].title);
						FEEDBAR.removeItem(items[i]);
					}
				}
			}
		}
	},
	
	showPreview : function (item) {
		var tt = this.previewPane;
		
		if (!item || !item.link){
			tt.style.visibility = 'hidden';
			return false;
		}
		else {
			while (document.getElementById("feedbarTooltipSummary").lastChild) document.getElementById("feedbarTooltipSummary").removeChild(document.getElementById("feedbarTooltipSummary").lastChild);
			
			var maxLength = 60;
			
			var descr = item.description;
			var title = item.title;
			var url = item.link;
			var feedName = item.feedName;
			
			if (title.length > maxLength){
				title = title.substring(0,maxLength) + "...";
			}
			
			if (url.length > maxLength){
				url = url.substring(0,maxLength) + "...";
			}
			
			var image = item.image;
			document.getElementById("feedbarTooltipImage").src = image;
			document.getElementById("feedbarTooltipURL").value = "URL: "+url;
			document.getElementById("feedbarTooltipFeedName").value = feedName;
			
			if (title == '') title = ' ';
			
			document.getElementById("feedbarTooltipName").value = title;
			document.getElementById("feedbarTooltipName").style.display = '';
					
			if (!descr){
				descr = "No summary provided.";
			}

			var text = document.createTextNode(descr);
			
			document.getElementById("feedbarTooltipSummary").appendChild(text);
	  
	  		tt.style.visibility = 'visible';
	  
			return true;
		}
	},
	
	addError : function (feedName, feedUrl, error, priority) {
		var nb = document.getElementById("sidebar-notify");
		nb.appendNotification(feedName + ": " + error, feedUrl, 'chrome://browser/skin/Info.png', priority, [ { accessKey : "V", callback : FEEDBAR.notifyCallback, label : "View feed", popup : null } ]);
	},

	notifyCallback : function (notification, description) {
		var browser = window.parent.gBrowser;
		
		var theTab = browser.addTab(notification.value);
		browser.selectedTab = theTab;
	},
	
	notifyNoFeeds : function () {
		var nb = document.getElementById("sidebar-notify");
		nb.appendNotification(this.strings.getString("feedbar.errors.noFeedsFound"), "noFeedsFound", 'chrome://browser/skin/Info.png', 5, [ { accessKey : this.strings.getString("feedbar.errors.noFeedsFoundKey"), callback : function () { FEEDBAR.noFeedsFoundCallback(); }, label : this.strings.getString("feedbar.errors.noFeedsFoundLabel"), popup : null } ]);
	},
	
	noFeedsFoundCallback : function () {
		alert(this.strings.getString("feedbar.errors.noFeedsFoundMore"));
	},
	
	clearNotify : function () {
		document.getElementById("sidebar-notify").removeAllNotifications();
	},
	
	getSelected : function () {
		var view = this.tree.view;
		var range = view.selection.getRangeCount();
		var items = [];

		for (var i = 0; i < range; i++) {
			var start = {};
			var end = {};
			view.selection.getRangeAt(i, start, end);
			return view.getItemAtIndex(start.value);
		}

		return null;
	},
	
	launchUrl : function (url, event) {
		if ((event.which == 2) || (event.which == 1 && (event.ctrlKey || event.metaKey))){
			this._addTab(url);
		}
		else if (event.which == 1){
			this._inTab(url);
		}
		else if (event.which == 4){
			window.open(url);
		}
	},
	
	_addTab : function (url) {
		var browser = window.parent.gBrowser || window.parent.gBrowser;
		var theTab = browser.addTab(url);
		
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
			
		var loadInBackground = false;
			
		try {
			loadInBackground = prefs.getBoolPref("browser.tabs.loadInBackground");
		} catch (e) {
		}
			
		if (!loadInBackground){
			browser.selectedTab = theTab;
		}
	},
	
	_inTab : function (url) {
		window.parent.content.document.location.href = url;
	}
};